#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from uuid import uuid4

# import interfaces
from pyams_content.interfaces import CREATE_CONTENT_PERMISSION
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_content.shared.site.interfaces import ISiteContainer, ISiteManager
from pyams_i18n.interfaces import II18nManager
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_workflow.interfaces import IWorkflowInfo, IWorkflowVersions
from pyams_zmi.layer import IAdminLayer
from zope.intid.interfaces import IIntIds

# import packages
from pyams_content.shared.common.zmi import SharedContentAddForm, SharedContentAJAXAddForm
from pyams_content.shared.site.zmi.widget import SiteManagerFoldersSelectorFieldWidget
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyramid.view import view_config
from z3c.form import field
from zope.interface import Interface
from zope.lifecycleevent import ObjectCreatedEvent
from zope.schema import Int
from pyams_content import _


@viewlet_config(name='add-topic.menu', context=ISiteContainer, layer=IAdminLayer, view=Interface,
                manager=IToolbarAddingMenu, permission=CREATE_CONTENT_PERMISSION, weight=20)
class TopicAddMenu(ToolbarMenuItem):
    """Topic add menu"""

    label = _("Add topic...")
    label_css_class = 'fa fa-fw fa-file-o'
    url = 'add-topic.html'
    modal_target = True


class ITopicAddFormFields(IWfSharedContent):
    """Topic add form fields interface"""

    parent = Int(title=_("Parent"),
                 description=_("Topic's parent"),
                 required=True)


@pagelet_config(name='add-topic.html', context=ISiteContainer, layer=IPyAMSLayer,
                permission=CREATE_CONTENT_PERMISSION)
@pagelet_config(name='add-shared-content.html', context=ISiteContainer, layer=IPyAMSLayer,
                permission=CREATE_CONTENT_PERMISSION)
class TopicAddForm(SharedContentAddForm):
    """Topic add form"""

    legend = _("Add topic")

    fields = field.Fields(ITopicAddFormFields).select('title', 'parent', 'notepad')
    fields['parent'].widgetFactory = SiteManagerFoldersSelectorFieldWidget

    ajax_handler = 'add-topic.json'
    edit_permission = CREATE_CONTENT_PERMISSION

    __target = None

    def updateWidgets(self, prefix=None):
        super(TopicAddForm, self).updateWidgets(prefix)
        if 'parent' in self.widgets:
            self.widgets['parent'].permission = CREATE_CONTENT_PERMISSION

    def create(self, data):
        manager = get_parent(self.context, ISiteManager)
        return manager.topic_content_factory.content_class()

    def update_content(self, content, data):
        # initialize content fields
        content.title = data['title']
        content.short_name = content.title.copy()
        content.notepad = data.get('notepad')
        content.creator = self.request.principal.id
        content.owner = self.request.principal.id
        # get parent
        intids = get_utility(IIntIds)
        parent = intids.queryObject(data.get('parent'))
        if parent is not None:
            languages = II18nManager(parent).languages
            if languages:
                II18nManager(content).languages = languages.copy()
            manager = get_parent(parent, ISiteManager)
            wf_parent = manager.topic_content_factory()
            self.request.registry.notify(ObjectCreatedEvent(wf_parent))
            uuid = str(uuid4())
            parent[uuid] = wf_parent
            IWorkflowVersions(wf_parent).add_version(content, None)
            IWorkflowInfo(content).fire_transition('init', comment=content.notepad)
            self.__target = content

    def add(self, content):
        pass

    def nextURL(self):
        return absolute_url(self.__target, self.request, 'admin')


@view_config(name='add-topic.json', context=ISiteContainer, request_type=IPyAMSLayer,
             permission=CREATE_CONTENT_PERMISSION, renderer='json', xhr=True)
class TopicAJAXAddForm(SharedContentAJAXAddForm, TopicAddForm):
    """Topic add form, JSON renderer"""
